/**
 *
 */
package eu.etaxonomy.taxeditor.navigation.navigator.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.conversation.ConversationHolderMock;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigatorLabels;
import eu.etaxonomy.taxeditor.navigation.navigator.operation.RemotingChangeAcceptedTaxonToSynonymOperation;
import eu.etaxonomy.taxeditor.operation.RemotingCdmHandler;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonNodeSelectionDialog;

/**
 * <p>ChangeAcceptedTaxonToSynonymHandler class.</p>
 *
 * @author n.hoffmann
 * @created Jan 4, 2010
 * @version 1.0
 */
public class RemotingChangeAcceptedTaxonToSynonymHandler extends RemotingCdmHandler {

    private static final Logger logger = Logger
            .getLogger(RemotingChangeAcceptedTaxonToSynonymHandler.class);


    private ITaxonTreeNode oldTaxonNode;

	/**
     * @param label
     */
    public RemotingChangeAcceptedTaxonToSynonymHandler() {
        super(TaxonNavigatorLabels.CHANGE_ACCEPTED_TAXON_TO_SYNONYM_LABEL);
    }
    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#allowOperations(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public IStatus allowOperations(ExecutionEvent event) {
        TreeSelection selection = (TreeSelection) HandlerUtil.getCurrentSelection(event);
        // check that only a single taxon tree node has been selected
        if(selection.size() > 1) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.SINGLE_TAXON_SELECTION_MESSAGE);
        }

        // check for no taxon tree node selected
        if(selection.size() == 0) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.NO_TAXON_SELECTION_MESSAGE);
        }

        // check that selected object is a taxon node
        Object obj = selection.iterator().next();
        if(obj instanceof ITaxonTreeNode) {
            oldTaxonNode = (ITaxonTreeNode)obj;
        } else {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.SELECTED_OBJECT_NOT_TREE_NODE_MESSAGE);
        }
        // check that the source taxon node does not have children
        if(((TaxonNode)oldTaxonNode).getCountChildren() > 0) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.SOURCE_TAXON_HAS_CHILDREN_MESSAGE);

        }

        // check if corresponding name editor is closed
        boolean editorClosed = NavigatorHandlerUtils.closeObsoleteEditor(event, (TaxonNode) oldTaxonNode);
        if(editorClosed != true) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.RELATED_EDITOR_NOT_CLOSED_MESSAGE);
        }

        return Status.OK_STATUS;
    }
    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#doOperations(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public AbstractOperation prepareOperation(ExecutionEvent event) {
        List<UUID> excludeTaxa = new ArrayList<UUID>();
        excludeTaxa.add(oldTaxonNode.getUuid());
        TaxonNode newAcceptedTaxonNode = TaxonNodeSelectionDialog.select(HandlerUtil.getActiveShell(event),
                new ConversationHolderMock(),
                "Choose the accepted taxon",
                excludeTaxa,
                null,
                ((TaxonNode)oldTaxonNode).getClassification());

        if (newAcceptedTaxonNode == null) {
            return null;
        }

        RemotingChangeAcceptedTaxonToSynonymOperation rcattso =
                new RemotingChangeAcceptedTaxonToSynonymOperation(event.getTrigger(),
                        false,
                        oldTaxonNode.getUuid(),
                        newAcceptedTaxonNode.getUuid());

        return rcattso;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#onComplete(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public void onComplete() {
        // TODO Auto-generated method stub

    }




}
