// $Id$
/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.navigation.navigator.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.conversation.ConversationHolderMock;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigatorLabels;
import eu.etaxonomy.taxeditor.navigation.navigator.operation.RemotingMoveTaxonOperation;
import eu.etaxonomy.taxeditor.operation.RemotingCdmHandler;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonNodeSelectionDialog;

/**
 * @author cmathew
 * @date 19 Jun 2015
 *
 */
public class RemotingMoveTaxonNodeHandler extends RemotingCdmHandler {

    private TaxonNode oldTaxonNode;

    public RemotingMoveTaxonNodeHandler() {
        super(TaxonNavigatorLabels.MOVE_TAXON_LABEL);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#allowOperations(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public IStatus allowOperations(ExecutionEvent event) {
        TreeSelection selection = (TreeSelection) HandlerUtil.getCurrentSelection(event);
        // check that only a single taxon tree node has been selected
        if(selection.size() > 1) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.SINGLE_TAXON_SELECTION_MESSAGE);
        }

        // check for no taxon tree node selected
        if(selection.size() == 0) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.NO_TAXON_SELECTION_MESSAGE);
        }
        // check that selected object is a taxon node
        Object obj = selection.iterator().next();
        if(obj instanceof TaxonNode) {
            oldTaxonNode = (TaxonNode)obj;
        } else {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.SELECTED_OBJECT_NOT_TREE_NODE_MESSAGE);
        }
        return Status.OK_STATUS;
    }


    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#prepareOperation(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public AbstractOperation prepareOperation(ExecutionEvent event) {
        Shell activeShell = HandlerUtil.getActiveShell(event);
        TaxonNode parentTaxonNode;

        List<UUID> excludeTaxa = new ArrayList<UUID>();
        excludeTaxa.add(oldTaxonNode.getUuid());

        boolean moveToNewParent = true;

        if (PreferencesUtil.getSortNodesNaturally()){
            if(!MessageDialog.openQuestion(activeShell, "Target node", "The choosen target node should be the parent?")){
                moveToNewParent = false;
            }
            parentTaxonNode = TaxonNodeSelectionDialog.select(activeShell,
                    new ConversationHolderMock(),
                    "Choose the taxon above the moved taxon.",
                    excludeTaxa,
                    null,
                    null);
        } else {
            parentTaxonNode = TaxonNodeSelectionDialog.select(activeShell,
                    new ConversationHolderMock(),
                    "Choose new parent",
                    excludeTaxa,
                    null,
                    null);
        }


        if(parentTaxonNode != null){
            if(NavigationUtil.isDirty(parentTaxonNode)){
                MessageDialog.openWarning(activeShell,
                        "Unsaved Parent Taxon",
                        "There are unsaved changes in the parent taxon. Please save first.");
                return null;
            }

            return new RemotingMoveTaxonOperation(event.getTrigger(),
                    false,
                    oldTaxonNode.getUuid(),
                    parentTaxonNode.getUuid(),
                    moveToNewParent);
        }

        return null;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#onComplete()
     */
    @Override
    public void onComplete() {
        // TODO Auto-generated method stub

    }

}
